// $Id: API.php,v 1.8 2006/04/05 02:58:50 morbus Exp $

/**
 * @file
 * API hooks are used by creating a function within your include file in the
 * form of "INCLUDENAME_HOOK". If you had "human.inc", for example, then
 * "human_info" would be the name of a function used by the "info" hook.
 *
 * API hooks:
 *  info   - Describes the data within your object (required).
 *  access - Determines who (player or character) can access your object.
 *
 * Each of the above API hooks also has a matching onHook, which receives
 * the exact same parameters as the hook, but also the name of the include
 * responding to the hook and the result of that response. For example, every
 * time an "info" hook is called and the result is received, that same result
 * is then passed to any "onInfo" hooks, along with the name of the include
 * that responded with that result. Consider the following:
 *
 *   - races/human.inc has a human_info(), which is an "info" hook.
 *   - plugins/example.inc has example_onInfo(), an "onInfo" hook.
 *
 * The following happens everytime an "info" hook is called:
 *
 *   - races/human.inc responds with the result of human_info().
 *   - plugins/example.inc receives "human" and the result in example_onInfo().
 *
 * This onHook system allows plugins (or other include files, though more 
 * rarely) to insert, intercept, and optionally modify results generated by
 * hooks. Function parameters are always "$result, $include, [HOOK_ARGS]"
 * where HOOK_ARGS are the same arguments received by the original hook.
 *
 * For examples of all the hooks in action, see plugins/example.inc.
 */

/** ---------------------------------------------------------------------------
 * info - Describes the data within your object (required).
 *
 * @return $object
 *   An object representing the data requested. Valid keys are:
 *
 *     - name (required): the human-showable name of the data within.
 *     - author (optional): the name of the person who created the data.
 *     - email (optional): email of the author who created the data.
 *     - url (optional): website to find more information about the data.
 *     - modified (optional): last date the data was modified, YYYY-MM-DD.
 *     - version (optional): a number that represents the current version.
 *
 *     - description_tiny, description_short, description_long (required):
 *         A description of the data within. This could be a description of
 *         a race, item, plugin, location, etc. The 'short' variant is used
 *         in listings, and the 'long' (which could be the same as 'short')
 *         is used on full-page views about the object. 'tiny', on the other
 *         hand, is usually ten words or less.
 */

/** ---------------------------------------------------------------------------
 * access - Determines who (player or character) can access your object.
 *
 * This hook determines how your object is accessible to both the player (i.e.,
 * visually in game text or in the Game Library) and the character (i.e., if
 * the character can use or "access" the object in some way). There are two
 * types of permission, "accessible by player" and "accessible by character",
 * which can either be 1 ("yes") or 0 ("no"). Together, these two permissions
 * create four possible access modes described below.
 *
 * Unfortunately, while these checks do happen during character creation, it
 * is technically and securely impossible to check the in-progress abilities
 * of the new character - thus, you can't expect to use game_mob_get() during
 * character creation to check against the Strength score, for example. From
 * a game standpoint, this makes it impossible to restrict race or class 
 * selection based on a new character's attribute. See the TODO.txt.
 *
 * accessible by player:     1
 * accessible by character:  1
 *   If your object falls under this access control, you DO NOT have to define
 *   the access hook in your include file - this is the default access mode.
 *
 *   - The player knows the object exists.
 *   - The object is accessible by the character.
 *   - EX: The player sees a sword, and the character can use it.
 *   - EX: The player is allowed to choose a class for his character.
 * 
 * accessible by player:     1
 * accessible by character:  0
 *   - The player knows the object exists.
 *   - The object in inaccessible by the character.
 *   - EX: The player and character can both see the sword, but the character
 *       is too weak to actually use it (they can still take it, but they just
 *       can't equip it or otherwise gain any benefits from it).
 *   - EX: The player can see there's a particular skill to choose, but his
 *       character does not have the required pre-requisites to obtain it.
 *
 * accessible by player:     0
 * accessible by character:  1
 *   Note: In most cases, this is an "oddball" permission, and should be used
 *   carefully. There are only rare instances this would actually be usable.
 *
 *   - The player doesn't know the object exists.
 *   - The object is accessible by the character.
 *   - EX: The player can not see anything in the room, but a character with
 *       the proper skill, and a successful contest, could see the hidden book.
 *
 * accessible by player:     0
 * accessible by character:  0
 *   - The player doesn't know the object exists.
 *   - The object is inaccessible to the character.
 *   - EX: Player and character can not see, or choose, a skill.
 *   - EX: Player and character can not see, or access, a hidden room.
 *
 * @param $type
 *   The permission to which access needs to be determined. This will either
 *   be the string "accessible by player" or "accessible by character".
 * @return $boolean
 *   Either 0 or 1, which represents the access of the $type requested.
 */

/** ---------------------------------------------------------------------------
 * attributes - Details the attributes of the game engine.
 *
 * This hook, primarily used by plugins/engine.inc, details which attributes
 * describe the game character and other objects within the Game API, as well
 * as their default values for a new game character. Default attributes are
 * split into "primary" and "secondary" groups, which are themed differently.
 * While other include files can create new primary and secondary attributes
 * (or replace them entirely), care should be given in regards to:
 *
 *   - that the new attribute is a "major" metric in the game. while
 *     plugins/alignment.inc can certainly be considered an attribute, it
 *     is not "major" enough, mechanic-wise, to be primary/secondary.
 *
 *   - that removing attributes will negatively affect any include file that
 *     depends on it in as-yet-undetermined ways. if you're gonna remove an
 *     attribute, or replace them all, it's a safe bet that all of your
 *     world data will have to be modified as well.
 *
 * The result of a hook_attributes call is expected to be an array of arrays,
 * with the keys being the internal attribute name, and indexes of the below.
 *
 *   - name: the string displayed to the user.
 *   - default_value: a number or string; ignored when unneeded.
 *   - abbreviation: an abbreviation of the attribute.
 *   - progression: the amount by which the attribute levels up.
 *   - description: a short description of what this attribute is.
 *   - group: whether the attribute is a 'primary' or 'secondary'.
 */

/** ---------------------------------------------------------------------------
 * game_character_node_facts - Returns strings related to the character.
 *
 * Primarily used by plugins to add facts about a character to the regular
 * node display. Return value is always an array of facts, generally one
 * sentence to one paragraph long. Facts are placed in an HTML unordered list.
 *
 * Since this is called only during a Drupal hook, this should really be in the
 * next block, but this file just needs a much better Doxygen/api.module
 * layout, so I'll move it there sometime in the future.
 */

/** ---------------------------------------------------------------------------
 * Drupal hooks - extending existing Drupal hooks to our include files.
 *
 * The following Drupal hooks have been extended for use in the include files.
 * Their usage is considered advanced, so we won't dwell on them long.
 *
 * miscellaneous functions:
 *
 *   drupal_settings: http://drupaldocs.org/api/head/function/hook_settings
 *     Called whenever admin/settings/game is accessed; sets configuration.
 *
 * game_character node functions:
 *
 *   drupal_access: http://drupaldocs.org/api/head/function/hook_access
 *     Called regarding operations on a game_character node. Primarily
 *     used in the GameAPI to control access to character creation based
 *     on game data (such as having too many characters, having one
 *     character of every possible race for balance issues, etc.) The
 *     result determines whether a player can even *start* creating a
 *     new character - do not use this hook to restrict races or classes,
 *     or other "after creation begins" effects (use the GameAPI access
 *     hook, above, for that).
 *
 *   drupal_load: http://drupaldocs.org/api/head/function/hook_load
 *     Called whenever a character is loaded for viewing or gameplay.
 *
 *   drupal_insert: http://drupaldocs.org/api/head/function/hook_insert
 *     Called when a new character is created. This is the primary place
 *     to add inventory or other "Hello!" sorts of things (such as an
 *     introductory quest, perhaps).
 *
 *   drupal_update: http://drupaldocs.org/api/head/function/hook_update
 *     Used only when the player has edited their character. This would
 *     only ever apply to your include file if a character's edit affects
 *     your data (see plugins/alignment.inc, which allows the character's
 *     "alignment" to be modified whenever a player likes.
 *
 *   drupal_delete: http://drupaldocs.org/api/head/function/hook_delete
 *     Called whenever a character is deleted. If you use the game_mob_set()
 *     functions to set your character data, then you won't ever need to
 *     worry about using this function.
 *
 *   drupal_view:
 *     Called during Drupal's NodeAPI "view" phase. Unlike regular Drupal
 *     modules, however, include files that respond to this hook should
 *     return ONLY the data they plan to APPEND to the existing node body.s
 *
 * Forms API related functions:
 *
 *   drupal_form_alter: http://drupaldocs.org/api/head/function/hook_form_alter
 *     Allows modification of any form that Drupal produces.
 *
 *   drupal_form_validate: equivalent to Form API's _validate capability
 *     Allows validation of any form, or form data, the user inputs.
 *
 *   drupal_form_pre_render: equivalent to Form API's #pre_render element.
 *     Allows modification of the form seconds before it is displayed.
 *
 *   drupal_form_submit: equivalent to Form API's _submit capability
 *     Occurs whenever a form is submitted - receives the submitted data.
 */
